#include "stdafx.h"
#include "netinfo.h"

using namespace net;

CString NetObjectEnumerator::GetErrorMessage(DWORD dwLastError) {
	HMODULE hModule = NULL;
	DWORD dwFormatFlags = FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_FROM_SYSTEM;
	LPTSTR lpMessageBuffer = NULL;
	if ((dwLastError >= NERR_BASE) && (dwLastError <= MAX_NERR)) {
		hModule = ::LoadLibraryEx(_T("netmsg.dll"), NULL, LOAD_LIBRARY_AS_DATAFILE);
		if (hModule != NULL)
			dwFormatFlags |= FORMAT_MESSAGE_FROM_HMODULE;
	}
	::FormatMessage(
		dwFormatFlags,
		hModule,
		dwLastError,
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		(LPTSTR)&lpMessageBuffer,
		0,
		NULL);
	CString szMessage;
	szMessage.Format(_T("%s (%u)"), lpMessageBuffer, dwLastError);
	::LocalFree(lpMessageBuffer);
	if (hModule != NULL)
		::FreeLibrary(hModule);
	return szMessage;
}

NetObjectEnumerator::NetObjectEnumerator() {
	index = 0;
}

NetObjectEnumerator::~NetObjectEnumerator() {
}

DWORD NetObjectEnumerator::GetError() const {
	return error;
}

INT_PTR NetObjectEnumerator::GetSize() const {
	return resources.GetSize();
}

bool NetObjectEnumerator::HasNext() const {
	return (index < resources.GetSize()) ? true : false;
}

NetObject * NetObjectEnumerator::GetNext() {
	return resources.GetAt(index++);
}

void NetObjectEnumerator::GoBegin() {
	index = 0;
}

NetObject * NetObjectEnumerator::GetAt(INT_PTR n) const {
	return resources.GetAt(n);
}

NetServerEnumerator::NetServerEnumerator(const CString &_domain) : NetObjectEnumerator() {
	domain = _domain;
	error = doQuery();
}

NetServerEnumerator::~NetServerEnumerator() {
	while (!resources.IsEmpty()) {
		delete (NetServer *)resources.GetAt(0);
		resources.RemoveAt(0);
	}
}

DWORD NetServerEnumerator::doQuery() {
	LPCWSTR pszDomain = NULL;
	LPSERVER_INFO_101 pBuffer, pTmpBuf = NULL;
	DWORD dwResult, dwCount, dwTotal, dwIndex = 0;
	if (!domain.IsEmpty())
		pszDomain = domain;
	dwResult = ::NetServerEnum(NULL, 101, (LPBYTE *)&pBuffer, MAX_PREFERRED_LENGTH, &dwCount, &dwTotal, SV_TYPE_NT, pszDomain, NULL);
	if ((dwResult == NERR_Success) || (dwResult == ERROR_MORE_DATA)) {
		pTmpBuf = pBuffer;
		for (dwIndex = 0; dwIndex < dwCount; dwIndex++) {
			NetServer *s = new NetServer();
			s->name.Format(_T("\\\\%s"), pTmpBuf->sv101_name);
			s->versionMajor = pTmpBuf->sv101_version_major;
			s->versionMinor = pTmpBuf->sv101_version_minor;
			s->platform = pTmpBuf->sv101_platform_id;
			s->type = pTmpBuf->sv101_type;
			s->comment = pTmpBuf->sv101_comment;
			resources.Add((NetObject *)s);
			pTmpBuf++;
		}
		::NetApiBufferFree(pBuffer);
	}
	return dwResult;
}

NetUserEnumerator::NetUserEnumerator(const CString &_server) : NetObjectEnumerator() {
	server = _server;
	// add leading backslashes ("\\") to the server name
	if ((!server.IsEmpty()) && (server.Find(_T("\\\\")) != 0))
		server.Insert(0, _T("\\\\"));
	error = doQuery();
}

NetUserEnumerator::~NetUserEnumerator() {
	while (!resources.IsEmpty()) {
		delete (NetUser *)resources.GetAt(0);
		resources.RemoveAt(0);
	}
}

DWORD NetUserEnumerator::doQuery() {
	LPCWSTR pszServer = NULL;
	PNET_DISPLAY_USER pBuffer, pTmpBuf = NULL;
	DWORD dwResult, dwCount, dwIndex = 0;
	if (!server.IsEmpty()) {
		pszServer = server;
		connectIPC();
	}
	do {
		dwResult = ::NetQueryDisplayInformation(pszServer, 1, dwIndex, 100, MAX_PREFERRED_LENGTH, &dwCount, (PVOID *)&pBuffer);
		if ((dwResult == ERROR_SUCCESS) || (dwResult == ERROR_MORE_DATA)) {
			pTmpBuf = pBuffer;
			for (; dwCount > 0; dwCount--) {
				NetUser *u = new NetUser();
				u->id = pTmpBuf->usri1_user_id;
				u->name = pTmpBuf->usri1_name;
				u->fullName = pTmpBuf->usri1_full_name;
				u->comment = pTmpBuf->usri1_comment;
				u->flags = pTmpBuf->usri1_flags;
				resources.Add((NetObject *)u);
				dwIndex = pTmpBuf->usri1_next_index;
				pTmpBuf++;
			}
			::NetApiBufferFree(pBuffer);
		}
	} while (dwResult == ERROR_MORE_DATA);
	if (!server.IsEmpty())
		disconnectIPC();
	return dwResult;
}

DWORD NetUserEnumerator::connectIPC() {
	CString strIPC;
	LMSTR pszIPC = NULL;
	int iSize = 0;
	USE_INFO_2 ui2Buf;
	DWORD dwRet = 0;
	strIPC.Format(_T("%s\\IPC$"), server);
	iSize = strIPC.GetLength() + 1;
	pszIPC = new TCHAR[iSize];
	_tcscpy_s(pszIPC, iSize, strIPC);
	::ZeroMemory(&ui2Buf, sizeof(USE_INFO_2));
	ui2Buf.ui2_remote = pszIPC;
	ui2Buf.ui2_username = _T("");
	ui2Buf.ui2_password = _T("");
	ui2Buf.ui2_domainname = _T("");
	ui2Buf.ui2_asg_type = USE_IPC;
	dwRet = ::NetUseAdd(NULL, 2, (LPBYTE)&ui2Buf, NULL);
	delete pszIPC;
	TRACE("NetUseAdd() = %u", dwRet);
	return dwRet;
}

DWORD NetUserEnumerator::disconnectIPC() {
	CString strIPC;
	LMSTR pszIPC = NULL;
	int iSize = 0;
	DWORD dwRet = 0;
	strIPC.Format(_T("%s\\IPC$"), server);
	iSize = strIPC.GetLength() + 1;
	pszIPC = new TCHAR[iSize];
	_tcscpy_s(pszIPC, iSize, strIPC);
	dwRet = ::NetUseDel(NULL, pszIPC, USE_NOFORCE);
	delete pszIPC;
	TRACE("NetUseDel() = %u", dwRet);
	return dwRet;
}
